#!/usr/bin/env python
###############################################################################
#
# Copyright (C) 2013 Wind River Systems, Inc.
#
# This code is licensed under the GPLv2.
#
###############################################################################
import os
import sys
import string
# specify the class look-up paths too...
sys.path.append('/usr/share/ovp-ott')
import getopt
import ovp_ott_kernel_analyser as KA
import ovp_ott_analysis_result as AR
import ovp_ott_console_reporter as CR
import ovp_ott_html_reporter as HR
import ovp_ott_error_codes as ERR


###############################################################################
### Constants, variables...
###############################################################################
BASEDIR = os.path.dirname(os.path.realpath(__file__))

RT_LATENCY_HOST_FILE=BASEDIR+"/in_data/rt-latency-host.cfg"
RT_LATENCY_GUEST_FILE=BASEDIR+"/in_data/rt-latency-guest.cfg"
RT_LATENCY_HOST_REPORT_TITLE="RT-LATENCY HOST"
RT_LATENCY_GUEST_REPORT_TITLE="RT-LATENCY GUEST"

RT_TUNE_HOST_FILE=BASEDIR+"/in_data/rt-tune-host.cfg"
RT_TUNE_GUEST_FILE=BASEDIR+"/in_data/rt-tune-guest.cfg"
RT_TUNE_HOST_REPORT_TITLE="RT-TUNE HOST"
RT_TUNE_GUEST_REPORT_TITLE="RT-TUNE GUEST"

PARAVIRT_KVM_FILE=BASEDIR+"/in_data/paravirt_kvm.cfg"

DPDK_DIR=BASEDIR+"/in_data/dpdk/"

###############################################################################
### Helper Methods...
###############################################################################

# Displays the tool help screen, and the command line switches + options.
def showHelp():
    helpStr = "USAGE:\n\t" + sys.argv[0] + " OPTIONS"
    helpStr += '''

Where OPTIONS:
    -h | --help \t- Shows this help screen
    -l | --latency \t- Check against rt-latency.
    -t | --tune \t- Check against rt-tune.
    -d | --dpdk \t- Check against DPDK tuning.
    -a | --all  \t- All the above options and parameters.
    -r | --report PARAM:<c|console h|html> - Specify report type.
    '''
    print string.center("*", 80, "*")
    print helpStr
    print string.center("*", 80, "*")


# opens the pInDir, and read every file in it into a list of lists.
# after that, this list of lists will be flattened, and compared against the
# kernel file content
def processDpdk(pInDir, pKernelLines, pKa, pAr):
    files = os.listdir(pInDir)
    clines = []
    for fileName in files:
        fullFileName = pInDir + fileName
        # read all cfg file content into one array of arrays
        clines.append(pKa.readFileIntoMemory(fullFileName))

    # flattern the array of arrays to one array
    pclines = [y for x in clines for y in x]
    # compare these with the current kernel config...
    pKa.compareWithKernelLines(pclines, pKernelLines, pAr, 'DPDK')


# Checks if the currently running system is configured as a Guest, or as a Host
# returns true if the current system is configured as GUEST.
#
# If all parameters from the paravirt_kvm.cfg file are present in the current
# kernel configuration, then the current system is Guest... otherwise it is a
# host system.
def isGuestInstance(ka, lines, ar):
    tempGroupName = "GTG"
    pv_lines = ka.readFileIntoMemory(PARAVIRT_KVM_FILE)
    ka.compareWithKernelLines(pv_lines, lines, ar, tempGroupName)
    rl = ar.getResultGroup(tempGroupName)
    ar.removeResultList(tempGroupName)
    # no elements in the temp results, means paravirt options are present in
    # the current kernel options = > we are running on a Guest build...
    if (len(rl) == 0):
        return True
    # by default we consider not a guest....
    return False


###############################################################################
### Main logic
###############################################################################
def main(argv):
    defaultReportType='console'
    try:
        opts, args = getopt.getopt(argv, "hltdar:",
        ["help", "latency", "tune", "dpdk", "all", "report"])
    except getopt.GetoptError:
        showHelp()
        sys.exit(ERR.OVP_OTT_ERR_CMDLINE_OPTION_ERROR)

    if len(opts) == 0:
        showHelp()
        sys.exit(ERR.OVP_OTT_ERR_INCOMPLETE_COMMANDLINE)


    for opt,arg in opts:
        # if -h is on cmd line, ignore everything else...
        if opt in ("-h", "--help"):
            showHelp()
            sys.exit(ERR.OVP_OTT_ERR_NO_ERROR)

    # check if we have raport type on the command line....
    for opt,arg in opts:
        if opt in ("-r", "--report"):
            if arg in ('c', 'console'):
                defaultReportType='console'
            elif arg in ('h', 'html'):
                defaultReportType='html'
            else:
                showHelp()
                sys.exit(ERR.OVP_OTT_ERR_CMDLINE_OPTION_ERROR)
            # remove this option, to not cause error in the loop through...
            opts.remove((opt,arg))

    # check if -a | --all is present in the cmd line...
    for opt,arg in opts:
        if opt in ("-a", "--all"):
            opts = []
            opts.append( ("--latency", "") )
            opts.append( ("--tune", "") )
            opts.append( ("--dpdk", "") )


    # create the analyser and the result objects... prepare (read) the
    # kernel config params for analysis
    ka = KA.OVP_OTT_KERNEL_ANALYSER()
    ar = AR.OVP_OTT_ANALYSIS_RESULT()
    lines = ka.readKernelConfig()
    isGuest = isGuestInstance(ka, lines, ar)

    # process each command line argument
    for opt,arg in opts:
        #### latency
        if opt in ("-l", "--latency"):
            if isGuest:
                clines = ka.readFileIntoMemory(RT_LATENCY_GUEST_FILE)
                ka.compareWithKernelLines(clines, lines, ar,
                                          RT_LATENCY_GUEST_REPORT_TITLE)
            else:
                clines = ka.readFileIntoMemory(RT_LATENCY_HOST_FILE)
                ka.compareWithKernelLines(clines, lines, ar,
                                          RT_LATENCY_HOST_REPORT_TITLE)
        #### tune
        elif opt in ("-t", "--tune"):
            if isGuest:
                clines = ka.readFileIntoMemory(RT_TUNE_GUEST_FILE)
                ka.compareWithKernelLines(clines, lines, ar,
                                          RT_TUNE_GUEST_REPORT_TITLE)
            else:
                clines = ka.readFileIntoMemory(RT_TUNE_HOST_FILE)
                ka.compareWithKernelLines(clines, lines, ar,
                                          RT_TUNE_HOST_REPORT_TITLE)
        #### dpdk
        elif opt in ("-d", "--dpdk"):
            processDpdk(DPDK_DIR, lines, ka, ar)
        else:
            # -a, -r, -h, could cause this, but they clear the options list...
            showHelp()
            sys.exit(ERR.OVP_OTT_ERR_COMMAND_ERROR)

    # create the report format....
    if defaultReportType == 'console':
        report = CR.OVP_OTT_CONSOLE_REPORTER()
    elif defaultReportType == 'html':
        report = HR.OVP_OTT_HTML_REPORTER()

    report.setReportName("OVP-OTT Kernel Parameters Report")
    report.setTodayReportDate()
    report.writeReport(ar)



if __name__ == "__main__":
    main(sys.argv[1:])

sys.exit(ERR.OVP_OTT_ERR_NO_ERROR)

